<?php
/**
 * API WPLMS CHAT
 *
 * @author 		VibeThemes
 * @category 	Init
 * @package 	vibe-timetracker/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}



class Vibe_TimeTracker_Api{


	public static $instance;

    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new Vibe_TimeTracker_Api();
        return self::$instance;
    }


    function __construct(){

    	add_action('rest_api_init',array($this,'register_routes'));
    }


    function register_routes(){
    	
        
        register_rest_route( VIBE_TIMETRACKER_API_NAMESPACE, '/savetimetracker', array(
            array(
                'methods'             =>  "POST",
                'callback'            =>  array( $this, 'savetimetracker' ),
                'permission_callback'       => array( $this, 'get_timetracker_members_permissions_check' ),
            ),
        ));

        register_rest_route( VIBE_TIMETRACKER_API_NAMESPACE, '/edittimetracker', array(
            array(
                'methods'             =>  "POST",
                'callback'            =>  array( $this, 'edittimetracker' ),
                'permission_callback'       => array( $this, 'get_timetracker_members_permissions_check' ),
            ),
        ));
        register_rest_route( VIBE_TIMETRACKER_API_NAMESPACE, '/deletetimetracker', array(
            array(
                'methods'             =>  "POST",
                'callback'            =>  array( $this, 'deletetimetracker' ),
                'permission_callback'       => array( $this, 'get_timetracker_members_permissions_check' ),
            ),
        ));
        
        
        register_rest_route( VIBE_TIMETRACKER_API_NAMESPACE, '/gettimetrackerlogs', array(
            array(
                'methods'             =>  "POST",
                'callback'            =>  array( $this, 'gettimetrackerlogs' ),
                'permission_callback'       => array( $this, 'get_timetracker_members_permissions_check' ),
            ),
        ));
        register_rest_route( VIBE_TIMETRACKER_API_NAMESPACE, '/getReportDetails', array(
            array(
                'methods'             =>  "POST",
                'callback'            =>  array( $this, 'getReportDetails' ),
                'permission_callback'       => array( $this, 'get_timetracker_members_permissions_check' ),
            ),
        ));
        register_rest_route( VIBE_TIMETRACKER_API_NAMESPACE, '/get/mytimetracked', array(
            array(
                'methods'             =>  "POST",
                'callback'            =>  array( $this, 'mytimetracked' ),
                'permission_callback'       => array( $this, 'get_timetracker_members_permissions_check' ),
            ),
        ));
        
        

    }


    function get_timetracker_members_permissions_check($request){
        $body = json_decode($request->get_body(),true);
            
        if(!empty($body['token'])){
            $this->user = apply_filters('vibebp_api_get_user_from_token','',$body['token']);
            if(!empty($this->user)){
                return true;
            }
        }

        return false;
    }

    function edittimetracker($request){
        $body = json_decode($request->get_body(),true);
        if(user_can($this->user->id,'manage_options') || (!empty($body['author_id']) && $body['author_id']==$this->user->id)){
            if(!empty($body) && !empty($body['content'])){
                unset($body['token']);
                $tracker = new VIBE_TRACKER_DB();
                $id = $tracker->update_tracker($body);
                if(!empty($id)){
                    return new WP_REST_Response( array('status'=>1,'message'=>_x('Time tracker edited successfully','','vibe-timetracker')), 200 );
                }
            }
            return new WP_REST_Response( array('status'=>0,'message'=>_x('Log could not be edited,some error occured','','vibe-timetracker')), 200 );
        }
        return new WP_REST_Response( array('status'=>0,'message'=>_x('You dont have permissions to edit the log','','vibe-timetracker')), 200 );
        

    }

    function deletetimetracker($request){
        $body = json_decode($request->get_body(),true);
        if(user_can($this->user->id,'manage_options') || (!empty($body['author_id']) && $body['author_id']==$this->user->id)){
            if(!empty($body) && !empty($body['tracker_id'])){
                unset($body['token']);
                $tracker = new VIBE_TRACKER_DB();
                $tracker->delete_tracker($body['tracker_id']);
                
                return new WP_REST_Response( array('status'=>1,'message'=>_x('Time tracker deleted successfully','','vibe-timetracker')), 200 );
                
            }
            return new WP_REST_Response( array('status'=>0,'message'=>_x('Log could not be deleted,some error occured','','vibe-timetracker')), 200 );
        }
        return new WP_REST_Response( array('status'=>0,'message'=>_x('You dont have permissions to delete the log','','vibe-timetracker')), 200 );

    }

    function savetimetracker($request){
        $response = array('status'=>0,'message'=>_x('Error saving!','','vibe-timetracker'));
        $body = json_decode($request->get_body(),true);
        if(!empty($body) && !empty($body['card_id'])){
            $card_id = intval($body['card_id']);
            $user_id  =$this->user->id;
            //use add_user_meta

            $fields = $body['fields'];
            $comment_content = '';
            if(!empty($fields)){
                if(!empty($fields['comment_form'])){
                    $comment_content = $fields['comment_form'];
                }

            }
            if(empty($comment_content)){
                return new WP_REST_Response( array('status'=>0,'message'=>_x('Time tracker description can\'t be empty','','vibe-timetracker')), 200 );
            }

            if(!empty($card_id)){
                $billable = 0;
                if(!empty($fields) && !empty($fields['billable'])){
                    $billable = 1;
                }
                $tt = array(
                    'type'=>'card',
                    'status' => 1,
                    'content'=>$comment_content,
                    'item_id'=>$card_id,
                    'author_id'=>$this->user->id,
                    'date'=>date('Y-m-d H:i:s'),
                    'end'=>intval($body['endTime']),
                    'start'=>intval($body['startTime']),
                    'billable'=>$billable
                );
                $tracker = new VIBE_TRACKER_DB();
                if(!empty($body['tracker_id'])){
                    $tid = intval($body['tracker_id']);
                    $tt['tracker_id'] = $tid;
                    unset($tt['author_id']);
                    $tracker->update_tracker($tt);
                }else{
                    $tid = $tracker->insert_tracker(apply_filters('vibe_card_time_details',
                    $tt,$request,$this->user));
                }
                

                //item_id = project id and secondary_item_id = Card ID

                

                
                if(!empty($tid)){
                    
                    do_action('vibe_projects_timetracker_added',$card_id,$tid,$tt,$this->user,$body);
                    $response['status'] = 1;
                    $response['message'] = _x('Time session saved successfully!','','vibe-timetracker');
                }
                
                
            }
        }
        return new WP_REST_Response( $response, 200 );
    }

    function gettimetrackerlogs($request){
        $response = array('status'=>0,'message'=>_x('No history!','','vibe-timetracker'));
        $args = json_decode($request->get_body(),true);
        if(!empty($args) && !empty($args['item_id'])){
            global $wpdb;
            unset($args['token']);
            $tracker = new VIBE_TRACKER_DB();
            $total = $tracker->get_trackers($args,true,true);
            $results = $tracker->get_trackers($args);
            if(!empty($results)){
                $response['message'] = '';
                $response['status'] =1;
                $response['logs'] =$results;
                $response['args'] =$args;
                $response['total'] = $total;
            }
        }
        return new WP_REST_Response( $response, 200 );

    }

    function getReportDetails($request){
        $response = array('status'=>0,'message'=>_x('No data!','','vibe-timetracker'));
        $args = json_decode($request->get_body(),true);
        if(!empty($args['item_id'])){
            $args['project'] = $args['item_id'];
        }
        if(!empty($args) && !empty($args['project'])){
            $project_id = intval($args['project']);
            $tab = sanitize_text_field($args['tab']);
            global $wpdb;

            switch($tab){
                case 'overview':
                    $vt = new VIBE_TRACKER_DB;

                    $total_seconds = $wpdb->get_var($wpdb->prepare("SELECT SUM(vt.end-vt.start) from {$wpdb->posts} AS p LEFT JOIN {$wpdb->postmeta} AS pm ON pm.post_id=p.ID LEFT JOIN {$vt->table_name} AS vt ON pm.post_id=vt.item_id WHERE p.post_status = 'publish' AND pm.meta_key='vibe_card_project' AND pm.meta_value=%d AND vt.type='card'",$project_id));
           

                    $total_members = $wpdb->get_results($wpdb->prepare("SELECT DISTINCT CONCAT(vt.author_id, '_', vt.item_id) AS Group1 from {$wpdb->posts} AS p LEFT JOIN {$wpdb->postmeta} AS pm ON pm.post_id=p.ID LEFT JOIN {$vt->table_name} AS vt ON pm.post_id=vt.item_id WHERE p.post_status = 'publish' AND pm.meta_key='vibe_card_project' AND pm.meta_value=%d AND vt.type='card' GROUP BY Group1",$project_id));

                    if(!empty($total_members)){
                        $total_members = count($total_members);
                    }
                    $data = array(
                        array(
                            'value'=>round(($total_seconds/3600),2),
                            'label'=>_x('Total time tracked','','vibe-timetracker'),
                            'sublabel'=>_x('Hours','','vibe-timetracker'),
                        ),
                        array(
                            'value'=>$total_members,
                            'label'=>_x('Logging times','','vibe-timetracker'),
                            'sublabel'=>_x('Unique Members','','vibe-timetracker'),
                        ),
                    );
                    $response['data'] =$data;
                    $response['status'] =1;
                    $response['message'] = '';
                break;
                case 'task':
                    $data= [];
                    $vt = new VIBE_TRACKER_DB;
                    $start = (intval($args['start'])/1000);
                    $end = (intval($args['end'])/1000);


                    $results = $wpdb->get_results($wpdb->prepare("SELECT SUM(vt.end-vt.start) AS time,DATE(vt.date ) DateOnly FROM {$wpdb->posts} AS p LEFT JOIN {$wpdb->postmeta} AS pm ON pm.post_id=p.ID LEFT JOIN {$vt->table_name} AS vt ON pm.post_id=vt.item_id WHERE p.post_status = 'publish' AND pm.meta_key='vibe_card_project' AND pm.meta_value=%d AND vt.type='card' AND (vt.date BETWEEN %s AND %s) GROUP BY DateOnly",$project_id,date('Y-m-d',$start),date('Y-m-d',$end)));
                    if(!empty($results)){
                        $data['data'] = [];
                        $data['labels'] = [];
                        foreach ($results as $key => $result) {
                            $data['data'][] =  round((intval($result->time)/3600),2);
                            $data['labels'][]=$result->DateOnly;
                        }
                        $response['data'] =$data;
                        $response['status'] =1;
                        $response['message'] ='';
                    }
                    

                break;


                case 'members':
                    $data= [];
                    $vt = new VIBE_TRACKER_DB;
                    $offset = ($args['page']-1)*$args['per_page'];
                    $results = $wpdb->get_results($wpdb->prepare("SELECT SUM(vt.end-vt.start) AS time,p.post_title as card_name,vt.author_id AS member_id,CONCAT(vt.author_id, '_', vt.item_id) AS Group1 from {$wpdb->posts} AS p LEFT JOIN {$wpdb->postmeta} AS pm ON pm.post_id=p.ID LEFT JOIN {$vt->table_name} AS vt ON pm.post_id=vt.item_id WHERE p.post_status = 'publish' AND pm.meta_key='vibe_card_project' AND pm.meta_value=%d AND vt.type='card' GROUP BY Group1,card_name,member_id LIMIT %d,%d",$project_id,$offset,$args['per_page']));
                   
                    if(!empty($results)){
                        $data['data'] = [];
                        foreach ($results as $key => $result) {
                            $data['data'][] = array(
                                'time'=>round((intval($result->time)/60),2),
                                'task_name'=>$result->card_name,
                                'member'=>bp_core_get_user_displayname($result->member_id),
                            );
                        }
                        
                        $data['columns'] = array(
                            array('field'=>'time','title'=>_x('Time Tracked(in minutes)','','vibe-timetracker')),
                            array('field'=>'task_name','title'=>_x('Task','','vibe-timetracker')),
                            array('field'=>'member','title'=>_x('Member','','vibe-timetracker')),
                        );
                        $response['data'] =$data;
                        $response['status'] =1;
                        $response['message'] ='';
                    }
                    

                break;
                case 'tracker':
                    $data= [];
                    $vt = new VIBE_TRACKER_DB;

                    $results = $wpdb->get_results($wpdb->prepare("SELECT SUM(vt.end-vt.start) AS time,p.post_title as card_name,vt.item_id from {$wpdb->posts} AS p LEFT JOIN {$wpdb->postmeta} AS pm ON pm.post_id=p.ID LEFT JOIN {$vt->table_name} AS vt ON pm.post_id=vt.item_id WHERE p.post_status = 'publish' AND pm.meta_key='vibe_card_project' AND pm.meta_value=%d AND vt.type='card' GROUP BY vt.item_id ORDER BY time DESC LIMIT 0,1",$project_id));
                 
                    if(!empty($results) && !empty($results[0])){
                        $row = $results[0];
                        if(!empty($row->item_id)){
                            $authors = [];
                            $users = $wpdb->get_results($wpdb->prepare("SELECT DISTINCT vt.author_id from {$vt->table_name} AS vt  WHERE vt.item_id=%d",$row->item_id));
                            if(!empty($users)){
                                foreach ($users as $key => $u) {
                                    $authors[]  = $u->author_id;
                                }
                            }
                        }
                        $data[] =array(
                            'type'=>'mega_card',
                            'title'=>_x('Card with most time tracked','','vibe-timetracker'),
                            'label'=>$row->card_name,
                            'sublabel'=>sprintf(_x('%s hours time tracked','','vibe-timetracker'),round(intval($row->time)/3600),3),
                            'authors'=>$authors
                        );
                        
                    }


                    $results2 = $wpdb->get_results($wpdb->prepare("SELECT SUM(vt.end-vt.start) AS time,vt.author_id,COUNT(DISTINCT vt.item_id) AS count from {$wpdb->posts} AS p LEFT JOIN {$wpdb->postmeta} AS pm ON pm.post_id=p.ID LEFT JOIN {$vt->table_name} AS vt ON pm.post_id=vt.item_id WHERE p.post_status = 'publish' AND pm.meta_key='vibe_card_project' AND pm.meta_value=%d AND vt.type='card' GROUP BY vt.author_id ORDER BY time DESC LIMIT 0,1",$project_id));

                    if(!empty($results2) && !empty($results2[0])){
                        $row = $results2[0];
                        
                        $data[] =array(
                            'type'=>'mega_member',
                            'title'=>_x('Member clocking max time','','vibe-timetracker'),
                            'label'=>bp_core_get_user_displayname($row->author_id),
                            'avatar'=>bp_core_fetch_avatar(array('item_id'=>$row->author_id,'object' => 'user')),
                            'sublabel'=>sprintf(_x('%s hours time tracked','','vibe-timetracker'),round(intval($row->time)/3600),3),
                        );
                        
                    }

                    if(!empty($data)){
                        $response['data'] =$data;
                        $response['status'] =1;
                        $response['message'] ='';
                    }
                    
                break;
                default:
                $response['data'] = apply_filters(array(),$project_id,$tab,$args);
                
                break;
            }

            
            
        }
        return new WP_REST_Response( $response, 200 );
    }

    function mytimetracked($request){
        $response = array('status'=>0,'message'=>_x('No data!','','vibe-timetracker'));
        $args = json_decode($request->get_body(),true);
        $user_id = $this->user->id;
        global $wpdb;
        $data = [];
        $date_format = get_option('date_format');
        $vt = new VIBE_TRACKER_DB;
        $start = (intval($args['start'])/1000);
        $end = (intval($args['end'])/1000);
       


        $results = $wpdb->get_results($wpdb->prepare("SELECT SUM(vt.end-vt.start) AS time,DATE(vt.date ) DateOnly FROM {$wpdb->posts} AS p  LEFT JOIN {$vt->table_name} AS vt ON p.ID=vt.item_id WHERE p.post_status = 'publish' AND vt.type='card' AND (vt.date BETWEEN %s AND %s) AND vt.author_id = %d  GROUP BY DateOnly",date('Y-m-d',$start),date('Y-m-d',$end),$user_id));

        if(!empty($results)){
            $data['data'] = [];
            $data['labels'] = [];
            foreach ($results as $key => $result) {
                $data['data'][] =  round((intval($result->time)/3600),2);
                $data['labels'][]=date($date_format,strtotime($result->DateOnly));
            }
            $response['status'] = 1;
            $response['data'] = $data;
            $response['message'] = '';
        }
        return new WP_REST_Response( $response, 200 );
    }
}

Vibe_TimeTracker_Api::init();