<?php
/**
 * PRofile
 *
 * @class       Vibe_Projects_Profile
 * @author      VibeThemes
 * @category    Admin
 * @package     vibecal
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Vibe_TimeTracker_Init{


	public static $instance;
	public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new Vibe_TimeTracker_Init();
        return self::$instance;
    }

	private function __construct(){
        add_filter('vibe_projects_enqueue_project_scripts',[$this,'add_card_time_tracker']);
        add_action('wp_enqueue_scripts',array($this,'load_tt_projects_scripts'),999999);
        add_filter('vibebp_custom_header_items',array($this,'vibebp_custom_header_items'),999999);
        add_filter('vibe_projects_enqueue_project_scripts',[$this,'add_timetracker_reports']);

        add_filter('vibe_projects_my_reports',[$this,'vibe_projects_my_reports_timetracker'],10,3);

        add_filter('vibe_invoicees_get_member_expense_details',[$this,'add_timetracked']);

    }

    function add_timetracked($expense){
        $expense['timetracked'] = $this->get_card_total_time_tracked($expense['card_id'],$expense['member']);
        return $expense;
    }

    function get_card_total_time_tracked($card_id,$member_id=null){
        global $wpdb;
        $vt = new VIBE_TRACKER_DB;
        if($member_id){
             $time = $wpdb->get_var($wpdb->prepare("SELECT SUM(vt.end-vt.start) AS time FROM {$vt->table_name} AS vt  WHERE vt.type='card' AND vt.item_id = %d AND vt.billable=1 AND vt.author_id=%d",$card_id,$member_id));
        }else{
             $time = $wpdb->get_var($wpdb->prepare("SELECT SUM(vt.end-vt.start) AS time FROM {$vt->table_name} AS vt  WHERE vt.type='card' AND vt.item_id = %d AND vt.billable=1 ",$card_id));
        }
        if(!empty($time)){
            return $time;
        }
        return 0 ;
    }

    function vibe_projects_my_reports_timetracker($rps,$user_id,$type){
        $rps[] = array(

            'title'=> __('Time tracked this week','vibe-projects'),
            'class'=>'',
            'type'=>'bar',
            'data'=> $this->get_cards_time_tracked($user_id,'week')
           
        );
        $rps[] = array(
            'title'=> __('Time tracked this month','vibe-projects'),
            'class'=>'',
            'type'=>'bar',
            'data'=> $this->get_cards_time_tracked($user_id,'month')
        );
        return $rps;
    }


    function get_cards_time_tracked($user_id,$range){
        $data= [];
        $vt = new VIBE_TRACKER_DB;
        global $wpdb;
        $date_format = get_option('date_format');
        switch($range){
            case 'week':

                $week = date('W', time());
                $dates = $this->getStartAndEndDate($week,date("Y"));
                $start = $dates['week_start'];
                $end = $dates['week_end'];


                $results = $wpdb->get_results($wpdb->prepare("SELECT SUM(vt.end-vt.start) AS time,DATE(vt.date ) DateOnly FROM {$wpdb->posts} AS p  LEFT JOIN {$vt->table_name} AS vt ON p.ID=vt.item_id WHERE p.post_status = 'publish' AND vt.type='card' AND (vt.date BETWEEN %s AND %s) AND vt.author_id = %d  GROUP BY DateOnly",date('Y-m-d',$start),date('Y-m-d',$end),$user_id));

                if(!empty($results)){
                    $data['data'] = [];
                    $data['labels'] = [];
                    foreach ($results as $key => $result) {
                        $data['data'][] =  round((intval($result->time)/3600),2);
                        $data['labels'][]=date($date_format,strtotime($result->DateOnly));
                    }
                }

            break;
            case 'month':
                $start = strtotime(date('Y-m-01'));
                $end = strtotime(date('Y-m-t'));


                $results = $wpdb->get_results($wpdb->prepare("SELECT SUM(vt.end-vt.start) AS time,WEEK(vt.date) DateOnly FROM {$wpdb->posts} AS p  LEFT JOIN {$vt->table_name} AS vt ON p.ID=vt.item_id WHERE p.post_status = 'publish' AND vt.type='card' AND (vt.date BETWEEN %s AND %s) AND vt.author_id = %d  GROUP BY DateOnly",date('Y-m-d',$start),date('Y-m-d',$end),$user_id));

                if(!empty($results)){
                    $data['data'] = [];
                    $data['labels'] = [];
                    foreach ($results as $key => $result) {
                        $data['data'][] =  round((intval($result->time)/3600),2);

                        $dates = $this->getStartAndEndDate($result->DateOnly,date("Y"));
                        $start = date($date_format,$dates['week_start']);
                        $end = date($date_format,$dates['week_end']);

                        $data['labels'][]=$start .' - '.$end;
                    }
                }
            break;
        }
        return $data;
        
    }

    function getStartAndEndDate($week, $year) {
      $dto = new DateTime();
      $dto->setISODate($year, $week);
      $ret['week_start'] = $dto->getTimestamp();
      $dto->modify('+6 days');
      $ret['week_end'] = $dto->getTimestamp();
      return $ret;
    }

    function add_timetracker_reports($settings){
        $settings['settings']['project_reports'][]=['value'=>'timetracker','label'=>__('Time Tracker','vibe-timetracker')];
        return $settings;
    }

    function vibebp_custom_header_items($divs){
        $divs[] = 'projects_timer';
        return $divs;
    }

    function add_card_time_tracker($args){

        $args['settings']['card_tabs'][]=['label'=>__('Time Tracker','vibe-timetracker'),'key'=>'timetracker'];
        
        return $args;
    }

    function load_tt_projects_scripts(){

        if(function_exists('bp_is_user') && bp_is_user() || apply_filters('vibebp_enqueue_profile_script',false)){
            $this->vibe_projects_timetracker();
         }
   
    }

    function vibe_projects_timetracker($args=null){

      
        $avatar ='';

        if(function_exists('vibe_get_option')){
            $avatar = vibe_get_option('default_avatar');
        }
        
        if(empty($avatar)){
            $avatar = plugins_url(__FILE__,'../assets/images/avatar.jpg');
        }
        $blog_id = '';
        if(function_exists('get_current_blog_id')){
            $blog_id = get_current_blog_id();
        }
        $data = array(
          
            'settings'=>array(
               'end_form_fields'=>array(
                    array(
                        'id'=>'comment_form',
                        'type'=>'textarea',
                        'label'=>_x('Description of session','','vibe-timetracker'),
                        'required'=>true
                    ),
                    array(
                        'id'=>'billable',
                        'type'=>'checkbox',
                        'label'=>_x('Billable','','vibe-timetracker'),
                    )
               ),
               'tracker_tabs'=>array(
                   'history'=>_x('History','','vibe-timetracker'),

               ),
               'logs_per_page'=>apply_filters('vibe_projects_timetracker_logs_per_page',10),
               'date_format'=>apply_filters('vibe_timetracker_date_format','MM/DD/YYYY'),
               'time_format'=>apply_filters('vibe_timetracker_time_format','HH:mmA'),
               'report_tabs'=>array(
                array(
                    'key'=>'overview',
                    'label'=>_x('Overview','','vibe-timetracker'),
                ),
                 array(
                    'key'=>'tasks',
                    'label'=>_x('Time tracked per day','','vibe-timetracker'),
                ),
                  array(
                    'key'=>'members',
                    'label'=>_x('Task and members','','vibe-timetracker'),
                ),
                   array(
                    'key'=>'tracker',
                    'label'=>_x('Time Tracked','','vibe-timetracker'),
                )
               ),
           ),
            'time_labels' => array(
                'year' => array('single'=>_x('year','time_labels','vibe-timetracker'),'multi'=>_x('years','time_labels','vibe-timetracker'),'symbol'=>_x('Y','time_labels','vibe-timetracker')),
                
                'month' => array('single'=>_x('month','time_labels','vibe-timetracker'),'multi'=>_x('months','time_labels','vibe-timetracker'),'symbol'=>_x('M','time_labels','vibe-timetracker')),
                'week' => array('single'=>_x('week','time_labels','vibe-timetracker'),'multi'=>_x('weeks','time_labels','vibe-timetracker'),'symbol'=>_x('W','time_labels','vibe-timetracker')),
                'day' => array('single'=>_x('day','time_labels','vibe-timetracker'),'multi'=>_x('days','time_labels','vibe-timetracker'),'symbol'=>_x('d','time_labels','vibe-timetracker')),
                'hour' => array('single'=>_x('hour','time_labels','vibe-timetracker'),'multi'=>_x('hours','time_labels','vibe-timetracker'),'symbol'=>_x('h','time_labels','vibe-timetracker')),
                'minute' => array('single'=>_x('minute','time_labels','vibe-timetracker'),'multi'=>_x('minutes','time_labels','vibe-timetracker'),'symbol'=>_x('m','time_labels','vibe-timetracker')),
                'second' => array('single'=>_x('second','time_labels','vibe-timetracker'),'multi'=>_x('seconds','time_labels','vibe-timetracker'),'symbol'=>_x('s','time_labels','vibe-timetracker')),
            ),
            'api_url'=>get_rest_url($blog_id,VIBE_TIMETRACKER_API_NAMESPACE),
            'translations'=>array(
                'delete_log'=>_x('Are you sure you want to delete this log','','vibe-timetracker'),
                'cancel'=>_x('Cancel','','vibe-timetracker'),
                'yes'=>_x('Yes','','vibe-timetracker'),
                'time_tracked'=>_x('Time tracked','','vibe-timetracker'),
                'search_member'=>_x('Search member','','vibe-timetracker'),
                'search_task'=>_x('Search task','','vibe-timetracker'),
                'billable'=>_x('Billable','','vibe-timetracker'),
                'load_more'=>_x('Load more','','vibe-timetracker'),
                'save'=>_x('Save','','vibe-timetracker'),
                'start'=>_x('Start','','vibe-timetracker'),
                'end'=>_x('End','','vibe-timetracker'),
                'is_required_field'=>_x('is required field','','vibe-timetracker'),
                
                
            ),
        );
        $data = apply_filters('vibe_projects_timetracker_data',$data);
        if(!empty($args)){

            if(!empty($args['translations'])){
                $args['translations'] = array_merge($data['translations'],$args['translations']);    
            }
            $data = array_merge($data,$args);
        }
        $data = apply_filters('vibe_projects_timetracker_after_data',$data);
        wp_enqueue_script('vibe_projects_timetracker',plugins_url('../assets/js/projetcstimetracker.js',__FILE__),array(),VIBEP_TIMETRACKER_VERSION,true);

        wp_localize_script('vibe_projects_timetracker','vibe_projects_timetracker',$data);
        wp_enqueue_style('vibe_projects_timetracker',plugins_url('../assets/css/projetcstimetracker.css',__FILE__),array(),VIBEP_TIMETRACKER_VERSION);
    }
}

Vibe_TimeTracker_Init::init();