<?php

if ( ! defined( 'ABSPATH' ) ) exit;

class VIBE_TRACKER_DB extends WPAP_TRACKER_DB {
/**
 * Get things started
 *
 * @access  public
 * @since   1.0
*/
public function __construct() {
	global $wpdb;
	$this->table_name  = $wpdb->prefix . 'vibe_timetracker';
	$this->primary_key = 'tracker_id';
	$this->version     = '1.0';
	$this->num_rows = array();
}
	
	/**
	 * Get columns and formats
	 *
	 * @access  public
	 * @since   1.0
	*/
	public function get_columns() {
		return array(
			'tracker_id'    => '%d',
			'start' 		=> '%d',
			'end' 		=> '%d',
			'content'       	=> '%s',
			'billable'    		=> '%d',
			'type'     			=> '%s',
			'item_id'          	=> '%d',
			'parent_tracker_id' => '%d',
			'status'      		=> '%s',
			'date'        		=> '%s',
			'author_id' => '%d'
		);
	}
	/**
	 * Get default column values
	 *
	 * @access  public
	 * @since   1.0
	*/
	public function get_column_defaults() {
		return array(
			'author_id' 		=> get_current_user_id(),
			'status'			=> 'open',
			'date'        		=> date( 'Y-m-d H:i:s' ),
		);
	}
	/**
	 * Retrieve orders from the database
	 *
	 * @access  public
	 * @since   1.0
	 * @param   array $args
	 * @param   bool  $count  Return only the total number of results found (optional)
	*/
	public function get_trackers( $args = array(), $count = false, $no_limit = false) {

		global $wpdb;
		$original_args = $args;
		$defaults = array(
			'number'       		=> 20,
			'offset'       		=> 0,
			'tracker_id' 	=> 0,
			'author_id' 		=> 0,
			'booker_id' 		=> 0,
			'start'       	=> 0,
			'end'         	=> 0,
			'type'     			=> '',
			'item_id'          	=> NULL,
			'status'       		=> '',
			'sort'				=> '',
			'orderby'      		=> 'tracker_id',
			'order'        		=> 'DESC',
			'join'				=> '',
			'range'				=> array()
		);
		$args  = wp_parse_args( $args, $defaults );

		

		if(!empty($this->cached[$this->multi_implode('-',$args)])){
			return $this->cached[$this->multi_implode('-',$args)];
		}
		

		if( $args['number'] < 1 ) {
			$args['number'] = 999999999999;
		}
		$where = '';
		$join ='';

		// specific referrals
		if( ! empty( $args['tracker_id'] ) ) {
			
			if( is_array( $args['tracker_id'] ) ) {
				$tracker_id = implode( ',', $args['tracker_id'] );
			} else {
				$tracker_id = intval( $args['tracker_id'] );
			}
			$where .= "WHERE `tracker_id` IN( {$tracker_id} ) ";
		}

		// specific referrals
		if( isset( $args['parent_tracker_id'] ) ) {
			if( is_array( $args['parent_tracker_id'] ) ) {
				$parent_tracker_id = implode( ',', $args['parent_tracker_id'] );
			} else if(is_numeric($args['parent_tracker_id'])){
				$parent_tracker_id = intval( $args['parent_tracker_id'] );
			} 
			if(empty($where)){
				$where .=' WHERE ';
			}else{
				$where .=' AND ';
			}

			if($args['parent_tracker_id'] === 'EXISTS'){
				$where .= " `parent_tracker_id` > 0 ";
			}else if ($args['parent_tracker_id'] == 0 ){
				$where .= " ( `parent_tracker_id` IS NULL OR `parent_tracker_id` = 0 )";
			}else{
				$where .= " `parent_tracker_id` IN( {$parent_tracker_id} ) ";
			}
			
		}

		if( ! empty( $args['not_in'] ) ) {

			if( empty( $where ) ) {
				$where .= " WHERE";
			} else {
				$where .= " AND";
			}

			if( is_array( $args['not_in'] ) ) {
				$not_tracker_id = implode( ',', $args['not_in'] );
			} else {
				$not_tracker_id = intval( $args['tracker_id'] );
			}
			$where .= " `tracker_id` NOT IN( {$not_tracker_id} ) ";
		}

		

		if( !empty( $args['author_id'] ) ) {
			if( empty( $where ) ) {
				$where .= " WHERE";
			} else {
				$where .= " AND";
			}
			if( is_array( $args['author_id'] ) ) {
				$where .= " `author_id` IN(" . implode( ",", $args['author_id'] ) . ") ";
			} else {
				$where .= " `author_id` = " . $args['author_id'] . " ";
			}
		}

		if( ! empty( $args['not_author'] ) ) {
			if( empty( $where ) ) {
				$where .= " WHERE";
			} else {
				$where .= " AND";
			}
			
			if( is_array( $args['not_author'] ) ) {
				$where .= " `author_id` NOT IN(" . implode( ",", $args['not_author'] ) . ") ";
			} else {
				$where .= " `author_id` != " . $args['not_author'] . " ";
			}
		}

		

		if( ! empty( $args['start'] ) && ! empty( $args['end'] ) && empty($args['range'])) {
			if( empty( $where ) ) {
				$where .= " WHERE";
			} else {
				$where .= " AND";
			}
			if($args['sort'] == 'on_and_upcoming' || $args['sort'] == 'upcoming'){
				$where .= " `end` >= " . $args['start'] . "AND `start` < " . $args['end'] . " ";
			}else if($args['sort'] == 'past'){
				$where .= " `start` < " . $args['start'] . "AND `end` < " . $args['end'] . " ";
			}else{
				$where .= "(( `start` >= " . $args['start'] . " AND `start` <=" . $args['end'] . " ) OR (`end` >= " . $args['start'] . " AND `end` <=" . $args['end'] . ") OR (".$args['start']." >= start AND ".$args['start']." <= end ) OR (".$args['end']." >= start AND ".$args['end']." <= end ))";
				
			}
		}else{
			if( ! empty( $args['start'] ) && empty($args['range'])) {
				//print_r($args['start']);
			
				if( empty( $where ) ) {
					$where .= " WHERE";
				} else {
					$where .= " AND";
				}

				if($args['sort'] == 'on_and_upcoming' || $args['sort'] == 'upcoming'){
					$where .= " `end` >= " . $args['start'] . " ";
				}else if($args['sort'] == 'past'){
					$where .= " `start` < " . $args['start'] . " ";
				}else{
					//if(! empty( $args['end'] ) ){
						//$where .= " `start` >= " . $args['start'] . " AND `start` <=" . $args['end'] . " ";
					//}else{
						$where .= " `start` >= " . $args['start'] . " ";
					//}
				}
				//print_r($where);
				
			}


			if( ! empty( $args['end'] )  && empty($args['range'])) {
				//print_r($args['end']);
				
				if( empty( $where ) ) {
					$where .= " WHERE";
				} else {
					$where .= " AND";
				}
				if($args['sort'] == 'past'){
					
					$where .= " `end` < " . $args['end'] . " ";
				}else if($args['sort'] == 'on_and_upcoming'  || $args['sort'] == 'upcoming'){
					
					$where .= " `start` < " . $args['end'] . " ";
				}else{

					//if(! empty( $args['start'] ) ){
						//$where .= " `end` >= " . $args['start'] . " AND `end` <=" . $args['end'] . " ";
					//}else{
						$where .= " `end` <= " . $args['end'] . " ";
					//}
					
				}
				//print_r($where);
			}
		}


		if( isset( $args['item_id'] ) && $args['item_id']!==null && $args['item_id'] != 'any') {
			if( empty( $where ) ) {
				$where .= " WHERE";
			} else {
				$where .= " AND";
			}
			if( is_array( $args['item_id'] ) ) {
				$where .= " `item_id` IN(" . implode( ",", $args['item_id'] ) . ") ";
			} else if($args['item_id'] == 'none'){
				$where .= " ( `item_id` =  0 OR `item_id` =  NULL )";
			}else{
				$where .= " `item_id` = " . $args['item_id'] . " ";
			}
		}

		if( ! empty( $args['type'] ) ) {
			if( empty( $where ) ) {
				$where .= " WHERE";
			} else {
				$where .= " AND";
			}
			if( is_array( $args['type'] ) ) {
				$where .= " `type` IN('" . implode( "','", $args['type'] ) . "') ";
			} else {
				$where .= " `type` = '" . $args['type'] . "' ";
			}
		}

		if( ! empty( $args['status'] )  && $args['status'] != 'any') {
			if( empty( $where ) ) {
				$where .= " WHERE";
			} else {
				$where .= " AND";
			}
			if( is_array( $args['status'] ) ) {
				$where .= " `status` IN('" . implode( "','", $args['status'] ) . "') ";
			} else {
				$where .= " `status` = '" . $args['status'] . "' ";
			}
		}
		
		if( ! empty( $args['date'] ) ) {
			if( is_array( $args['date'] ) ) {
				if( ! empty( $args['date']['start'] ) ) {
					if( false !== strpos( $args['date']['start'], ':' ) ) {
						$format = 'Y-m-d H:i:s';
					} else {
						$format = 'Y-m-d 00:00:00';
					}
					$start = date( $format, strtotime( $args['date']['start'] ) );
					if( ! empty( $where ) ) {
						$where .= " AND `date` >= '{$start}'";
					} else {
						$where .= " WHERE `date` >= '{$start}'";
					}
				}
				if( ! empty( $args['date']['end'] ) ) {
					if( false !== strpos( $args['date']['end'], ':' ) ) {
						$format = 'Y-m-d H:i:s';
					} else {
						$format = 'Y-m-d 23:59:59';
					}
					$end = date( $format, strtotime( $args['date']['end'] ) );
					if( ! empty( $where ) ) {
						$where .= " AND `date` <= '{$end}'";
					} else {
						$where .= " WHERE `date` <= '{$end}'";
					}
				}
			} else {
				$year  = date( 'Y', strtotime( $args['date'] ) );
				$month = date( 'm', strtotime( $args['date'] ) );
				$day   = date( 'd', strtotime( $args['date'] ) );
				if( empty( $where ) ) {
					$where .= " WHERE";
				} else {
					$where .= " AND";
				}
				$where .= " $year = YEAR ( date ) AND $month = MONTH ( date ) AND $day = DAY ( date )";
			}
		}
		if(!empty($args['join'])){

			if(!empty($args['join']['from'])){
				$this->table_name .= ' as tn LEFT JOIN '.sprintf($args['join']['from'],' ON tn.'.$args['join']['on_p'].' = '.$args['join']['on_c']);
				
			}

			if(!empty($args['join']['where'])){
				$where .=' AND '.$args['join']['where'];
			}
		}

		if(!empty($args['range'])){
			//Here start and end of the ranges are correct
			//print_r($args);

			if( empty( $where ) ) {
				$where .= " WHERE";
			} else {
				$where .= " AND";
			}

		
			$date_ranges = array();
			

			$where .=' (';
			$date_range_added_inquery = true;
			foreach($date_ranges as $range){
				$keys = array_keys($range);
				foreach($keys as $key){
					if(strpos($key, 'start') !== false ){
						if($date_range_added_inquery){
							$where .= ' ( '.$key.' >= '.$range[$key];
							$date_range_added_inquery = false;
						}else{
							$where .= ' OR ( '.$key.' >= '.$range[$key];
						}
								
					}else{
						$where .= ' AND '.$key.' <= '.$range[$key].' )';
					}
				}
				
			}

			$time_range_added_inquery = true;
			$where .=')';
		}
		

		$args['orderby'] = ! array_key_exists( $args['orderby'], $this->get_columns() ) ? $this->primary_key : $args['orderby'];


		if(!empty($join)){
			$args['orderby'] =' A.'.$args['orderby'];
		}

		$args['orderby'] = ' ORDER BY '.$args['orderby'];
		
		if(!empty($args['group_by'])){
			$args['orderby'] = 'GROUP BY '.$args['group_by'].$args['orderby'];
		}

			
		if ( true === $count ) {
			//print_R('#');
			if($this->primary_key == 'tracker_id'){
				$this->primary_key = 'A.tracker_id';
			}

			if(empty($join)){
				$join .= " as A ";
			}
			if(true===$no_limit){
				$results = absint( $wpdb->get_var("SELECT COUNT({$this->primary_key}) FROM {$this->table_name} {$join} {$where} {$args['orderby']} {$args['order']} " ) );
			}else{
				$results = absint( 
				$wpdb->get_var(
					$wpdb->prepare("SELECT COUNT({$this->primary_key}) FROM {$this->table_name} {$join} {$where} {$args['orderby']} {$args['order']} LIMIT %d, %d;",absint( $args['offset'] ),absint( $args['number'] ) )));
			}
			
		} else {
			if(empty($join)){
				$select = '*';
			}else{
				$select = 'A.*';
			}
			$query = $wpdb->prepare(
					"SELECT {$select} FROM {$this->table_name} {$join} {$where} {$args['orderby']} {$args['order']} LIMIT %d, %d;",absint( $args['offset'] ),absint( $args['number'] )
				);
			
			//print_R($query);
			$results = $wpdb->get_results(
				$query,ARRAY_A
			);
			$this->num_rows[$this->multi_implode('-',$original_args)] = $wpdb->num_rows;

		}


		$this->cached[$this->multi_implode('-',$args)] = $results;

		return $results;
	}

	/**
	 * Retrieves the number of results found for a given query.
	 *
	 * @access public
	 * @since  1.0
	 *
	 * @param array $args Optional. Any valid get_trackers() arguments. Default empty array.
	 * @return int Number of trackers found for the given arguments.
	 */
	public function count( $args = array() ) {
		return $this->get_trackers( $args, true );
	}

	public function num_rows($args = array()){
		if(!empty($this->num_rows[$this->multi_implode('-',$args)]))
		return $this->num_rows[$this->multi_implode('-',$args)];
	}

	
	/**
	 * Create the table
	 *
	 * @access  public
	 * @since   1.0
	*/
	public function insert_tracker( $data = array()){
		$defaults = array(
			'author_id' 		=> get_current_user_id(),
			'type'     			=> 'open',
			'status'      		=> 'open',

		);
		$args = wp_parse_args( $data, $defaults );
		if ( empty( $args['date'] ) ) {
			unset( $args['date'] );
		} else {
			$time = strtotime( $args['date'] );
			$args['date'] = gmdate( 'Y-m-d H:i:s' );
		}

		$add = $this->insert( $args, 'tracker' );
		if ( $add ) {
			/**
			 * Fires immediately after a tracker has been added to the database.
			 *
			 * @param array $add The tracker data being added.
			 */
			do_action( 'vibe_tracker_insert_tracker', $add );
			return $add;
		}
		return false;
	}
	/**
	 * Create the table
	 *
	 * @access  public
	 * @since   1.0
	*/
	public function update_tracker( $data = array()){
		$defaults = array(
			
		);
		$args = wp_parse_args( $data, $defaults );
		
		$add = $this->update($args['tracker_id'],$args);
		if ( $add ) {
			/**
			 * Fires immediately after a tracker has been added to the database.
			 *
			 * @param array $add The tracker data being added.
			 */
			do_action( 'vibe_tracker_update_tracker', $add );
			return $add;
		}
		return false;
	}
	/**
	 * Create the table
	 *
	 * @access  public
	 * @since   1.0
	*/
	public function delete_tracker( $tracker_id){
		if(empty($tracker_id))
			return false;

		$this->delete($tracker_id);
		
	}

	/**
	 * Checks if an tracker exists.
	 *
	 * @access public
	 * @since  1.0
	 *
	 * @return bool True if the affiliate exists, otherwise false.
	*/
	public function tracker_exists( $args ) {
		global $wpdb;
		
		$exists = $wpdb->query( $wpdb->prepare("SELECT 1 FROM {$this->table_name} WHERE %s", $args));
		return ! empty( $exists );
	}
	/**
	 * Create the table
	 *
	 * @access  public
	 * @since   1.0
	*/
	public function create_table() {
		global $wpdb;
		require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
		$sql = "CREATE TABLE " . $this->table_name . " (
		tracker_id bigint(20) NOT NULL AUTO_INCREMENT,
		start bigint(20) NOT NULL,
		end bigint(20) NOT NULL,
		author_id bigint(20) NOT NULL,
		content longtext,
		billable bigint(20),
		type varchar(30) NULL,
		item_id bigint(20) NULL,
		status varchar(30) NOT NULL,
		parent_tracker_id bigint(20) NULL,
		date datetime NOT NULL,
		PRIMARY KEY  (tracker_id)
		) CHARACTER SET utf8 COLLATE utf8_general_ci;";

		dbDelta( $sql );
		update_option( $this->table_name . '_db_version', $this->version );
	}

	function multi_implode($glue,$array ) {
	    $ret = '';
	    if(!empty($array) && is_array($array)){
	    	foreach ($array as $item) {
		        if (is_array($item)) {
		            $ret .= $this->multi_implode($glue,$item) . $glue;
		        } else {
		            $ret .= $item . $glue;
		        }
		    }
	    }
	    

	    $ret = substr($ret, 0, 0-strlen($glue));

	    return $ret;
	}
}